@@ -3,7 +3,7 @@
  * 
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: arp-bsd.c,v 1.1.1.1 2001/10/11 04:14:52 dugsong Exp $
+ * $Id: arp-bsd.c,v 1.3 2001/10/15 07:52:17 dugsong Exp $
  */
 
 #include "config.h"
@@ -14,6 +14,10 @@
 #ifdef HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
 #endif
+#ifdef HAVE_STREAMS_ROUTE
+#include <sys/stream.h>
+#include <sys/stropts.h>
+#endif
 
 #include <net/if.h>
 #include <net/if_dl.h>
@@ -22,6 +26,7 @@
 #include <netinet/if_ether.h>
 
 #include <errno.h>
+#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -48,7 +53,11 @@ arp_open(void)
 	if ((a = malloc(sizeof(*a))) == NULL)
 		return (NULL);
 
+#ifdef HAVE_STREAMS_ROUTE
+	if ((a->fd = open("/dev/route", O_RDWR, 0)) < 0) {
+#else
 	if ((a->fd = socket(PF_ROUTE, SOCK_RAW, 0)) < 0) {
+#endif
 		free(a);
 		return (NULL);
 	}
@@ -65,6 +74,9 @@ arp_msg(arp_t *a, struct arpmsg *msg)
 	msg->rtm.rtm_version = RTM_VERSION;
 	msg->rtm.rtm_seq = ++a->seq; 
 	
+#ifdef HAVE_STREAMS_ROUTE
+	return (ioctl(a->fd, RTSTR_SEND, &msg->rtm));
+#else
 	if (write(a->fd, msg, msg->rtm.rtm_msglen) < 0) {
 		if (errno != ESRCH || msg->rtm.rtm_type != RTM_DELETE)
 			return (-1);
@@ -80,6 +92,7 @@ arp_msg(arp_t *a, struct arpmsg *msg)
 		return (-1);
 	
 	return (0);
+#endif
 }
 
 int
@@ -88,9 +101,8 @@ arp_add(arp_t *a, struct addr *pa, struct addr *ha)
 	struct arpmsg msg;
 	struct sockaddr_in *sin;
 	struct sockaddr *sa;
-#ifdef HAVE_SOCKADDR_SA_LEN
 	int index, type;
-#endif
+	
 	if (a == NULL || pa == NULL || ha == NULL) {
 		errno = EINVAL;
 		return (-1);
@@ -124,7 +136,6 @@ arp_add(arp_t *a, struct addr *pa, struct addr *ha)
 			return (-1);
 		}
 	}
-#ifdef HAVE_SOCKADDR_SA_LEN
 	if (sa->sa_family != AF_LINK) {
 		errno = EADDRNOTAVAIL;
 		return (-1);
@@ -132,9 +143,11 @@ arp_add(arp_t *a, struct addr *pa, struct addr *ha)
 		index = ((struct sockaddr_dl *)sa)->sdl_index;
 		type = ((struct sockaddr_dl *)sa)->sdl_type;
 	}
-#endif
 	if (addr_ntos(pa, (struct sockaddr *)sin) < 0 || addr_ntos(ha, sa) < 0)
 		return (-1);
+
+	((struct sockaddr_dl *)sa)->sdl_index = index;
+	((struct sockaddr_dl *)sa)->sdl_type = type;
 	
 	memset(&msg.rtm, 0, sizeof(msg.rtm));
 	msg.rtm.rtm_type = RTM_ADD;
@@ -143,9 +156,6 @@ arp_add(arp_t *a, struct addr *pa, struct addr *ha)
 	msg.rtm.rtm_flags = RTF_HOST | RTF_STATIC;
 #ifdef HAVE_SOCKADDR_SA_LEN
 	msg.rtm.rtm_msglen = sizeof(msg.rtm) + sin->sin_len + sa->sa_len;
-	
-	((struct sockaddr_dl *)sa)->sdl_index = index;
-	((struct sockaddr_dl *)sa)->sdl_type = type;
 #else
 	msg.rtm.rtm_msglen = sizeof(msg.rtm) + sizeof(*sin) + sizeof(*sa);
 #endif
@@ -192,12 +202,10 @@ arp_delete(arp_t *a, struct addr *pa)
 			return (-1);
 		}
 	}
-#ifdef HAVE_SOCKADDR_SA_LEN
 	if (sa->sa_family != AF_LINK) {
 		errno = ESRCH;
 		return (-1);
 	}
-#endif
 	msg.rtm.rtm_type = RTM_DELETE;
 	
 	return (arp_msg(a, &msg));
